import { PromiseConfirmationService } from '@/utils';
import {
  Cartesian2,
  Cartesian3,
  Color,
  MakeBillboardLabel,
  NearFarScalar,
  SpaceCatalogDataSource,
  SpaceEntity,
  VerticalOrigin,
  type Viewer,
  viewerReferenceFrameMixin,
} from 'orbpro';

declare global {
  interface Window {
    spaceCatalog: SpaceCatalogDataSource;
  }
}

export async function demoOrbitGeneration(viewer: Viewer) {
  // @ts-expect-error globalThis
  globalThis.spaceCatalog = new SpaceCatalogDataSource({ name: 'celestrak' });
  viewer.extend(viewerReferenceFrameMixin);
  viewer.referenceFrame = 1;

  // https://spacedatastandards.org/#/standards?search=OMM
  const ISS = new SpaceEntity({
    id: '25544',
    label: {
      backgroundColor: new Color(0.1, 0.1, 0.1, 0.9),
      font: `1rem Helvetica`,
      pixelOffset: new Cartesian2(10, 0),
      pixelOffsetScaleByDistance: new NearFarScalar(1.5e2, 3, 1.5e7, 0.5),
      scaleByDistance: new NearFarScalar(1.5e2, 1.5, 13e7, 0),
      show: false,
      showBackground: true,
    },
    name: 'ISS',
    point: {
      pixelSize: 1,
    },
    viewFrom: new Cartesian3(-1_678_500.749_350_715_4, -17_680_994.634_034_64, 24_667_690.486_357_275),
  });

  ISS.position.loadOMM({
    /* CCSDS OMM版本 */ ARG_OF_PERICENTER: 149.2285,
    /* 创建日期(ISO 8601 UTC格式) */ BSTAR: 0.000_282_17,
    /* 创建者 */ CCSDS_OMM_VERS: 0,
    /* 卫星名称 */ CENTER_NAME: null,
    /* 国际标识符(YYYY-NNNAAA) */ CLASSIFICATION_TYPE: 'U',
    /* 中心名称(例如:EARTH, MARS) */ COMMENT: null,
    /* 参考坐标系 */ COV_REFERENCE_FRAME: 23,
    /* 参考坐标系历元(ISO 8601 UTC格式) */ CREATION_DATE: null,
    /* 时间系统[M, UTC] */ CX_DOT_X: 0,
    /* 平均元素理论 */ CX_DOT_X_DOT: 0,
    /* 注释 */ CX_DOT_Y: 0,
    /* 平均开普勒根数的历元(ISO 8601 UTC格式) */ CX_DOT_Z: 0,
    /* 半长轴(km) */ CX_X: 0,
    /* 平均运动(圈/天) */ CY_DOT_X: 0,
    /* 偏心率(无量纲) */ CY_DOT_X_DOT: 0,
    /* 轨道倾角(度) */ CY_DOT_Y: 0,
    /* 升交点赤经(度) */ CY_DOT_Y_DOT: 0,
    /* 近地点幅角(度) */ CY_DOT_Z: 0,
    /* 平近点角(度) */ CY_X: 0,
    /* 引力常数(km³/s²) */ CY_Y: 0,
    /* 质量(kg) */ CZ_DOT_X: 0,
    /* 太阳辐射面积(m²) */ CZ_DOT_X_DOT: 0,
    /* 太阳辐射压系数(无量纲) */ CZ_DOT_Y: 0,
    /* 大气阻力面积(m²) */ CZ_DOT_Y_DOT: 0,
    /* 大气阻力系数(无量纲) */ CZ_DOT_Z: 0,
    /* 星历类型(SGP,SGP4等) */ CZ_DOT_Z_DOT: 0,
    /* 分类类型,默认'U' */ CZ_X: 0,
    /* NORAD编号 */ CZ_Y: 0,
    /* 根数组编号 */ CZ_Z: 0,
    /* 历元时的圈次 */ DRAG_AREA: 0,
    /* 气阻系数(1/地球半径) */ DRAG_COEFF: 0,
    /* 平均运动一阶导数(圈/天²) */ ECCENTRICITY: 0.000_334_9,
    /* 平均运动二阶导数(圈/天³) */ ELEMENT_SET_NO: 999,
    /* 协方差矩阵参考坐标系 */ EPHEMERIS_TYPE: 0,
    /* --- 位置/速度协方差矩阵(6x6下三角) --- */
    /* 位置协方差矩阵元素(km²) */ EPOCH: '2024-05-08T19:52:52.426848',
    /* 位置协方差矩阵元素(km²) */ GM: 0,
    /* 位置协方差矩阵元素(km²) */ INCLINATION: 51.6355,
    /* 位置协方差矩阵元素(km²) */ MASS: 0,
    /* 位置协方差矩阵元素(km²) */ MEAN_ANOMALY: 210.8902,
    /* 位置协方差矩阵元素(km²) */ MEAN_ELEMENT_THEORY: 0,
    /* 速度-位置协方差矩阵元素(km²/s) */ MEAN_MOTION: 15.510_256_15,
    /* 速度-位置协方差矩阵元素(km²/s) */ MEAN_MOTION_DDOT: 0,
    /* 速度-位置协方差矩阵元素(km²/s) */ MEAN_MOTION_DOT: 0.000_162_75,
    /* 速度协方差矩阵元素(km²/s²) */ NORAD_CAT_ID: 25_544,
    /* 速度-位置协方差矩阵元素(km²/s) */ OBJECT_ID: '1998-067A',
    /* 速度-位置协方差矩阵元素(km²/s) */ OBJECT_NAME: 'ISS (ZARYA)',
    /* 速度-位置协方差矩阵元素(km²/s) */ ORIGINATOR: null,
    /* 速度协方差矩阵元素(km²/s²) */ RA_OF_ASC_NODE: 150.5366,
    /* 速度协方差矩阵元素(km²/s²) */ REFERENCE_FRAME: 2,
    /* 速度-位置协方差矩阵元素(km²/s) */ REFERENCE_FRAME_EPOCH: null,
    /* 速度-位置协方差矩阵元素(km²/s) */ REV_AT_EPOCH: 45_244,
    /* 速度-位置协方差矩阵元素(km²/s) */ SEMI_MAJOR_AXIS: 0,
    /* 速度协方差矩阵元素(km²/s²) */ SOLAR_RAD_AREA: 0,
    /* 速度协方差矩阵元素(km²/s²) */ SOLAR_RAD_COEFF: 0,
    /* 速度协方差矩阵元素(km²/s²) */ TIME_SYSTEM: 11,
    /* --- 用户自定义字段 --- */
    /* 用户自定义BIP-0044类型 */ USER_DEFINED_BIP_0044_TYPE: 0,
    /* 用户自定义对象标识符 */ USER_DEFINED_EARTH_MODEL: null,
    /* 用户自定义地球模型 */ USER_DEFINED_EPOCH_TIMESTAMP: 0,
    /* 用户自定义历元时间戳 */ USER_DEFINED_MICROSECONDS: 0,
    /* 用户自定义微秒数 */ USER_DEFINED_OBJECT_DESIGNATOR: null,
  });

  ISS.point!.pixelSize = 10 as any;
  ISS.point!.color = Color.WHITE as any;

  MakeBillboardLabel({
    entity: ISS,
    fontSize: 36,
    text: 'ISS-text',
    verticalOrigin: VerticalOrigin.CENTER,
  });

  const issDataSource = new SpaceCatalogDataSource({ name: 'issSource' });
  Object.assign(globalThis, { issDataSource });
  issDataSource.entities.add(ISS); // FIXME: add后有问题。
  await viewer.dataSources.add(issDataSource);

  // await new Promise((_r) => setTimeout(_r, 1000));
  // viewer.camera.flyTo({
  //   destination: ISS.position.getValue(viewer.clock.currentTime)!,
  //   orientation: {
  //     heading: 0.0,
  //     pitch: -Math.PI / 2,
  //     roll: 0.0,
  //   },
  //   duration: 2,
  // });

  viewer.extend(viewerReferenceFrameMixin);
  viewer.referenceFrame = 1;

  const shshowOrbitow = await PromiseConfirmationService({ message: '要显示轨道吗？' });
  ISS.showOrbit({ show: shshowOrbitow });

  const showCoverage = await PromiseConfirmationService({ message: '要显示覆盖区吗？' });
  ISS.showCoverage({ show: showCoverage });

  // /* let dynamicTimeline = */ new DynamicTimeline(viewer.timeline.container, viewer);

  const track = await PromiseConfirmationService({ message: '要追踪卫星吗？' });
  if (track) viewer.trackedEntity = ISS;
}
